@@ -3,12 +3,9 @@ from .align import *
 from .set_similarity import *
 from .vsm import *
 
-try:
-    import pyemd
+from .. import pyemd
+if pyemd is not None:
     from .wmd import get_wmd, get_wmd_histograms
-except ModuleNotFoundError:
-    import warnings
-    warnings.warn("Couldn't import `pyemd` module")
     
 from .pairwise_chunked import pairwise_kernels_chunked
 from .transportation import parallel_linear_sum_assignment
