@@ -6,20 +6,16 @@ Inspired by:
 """
 
 import logging
+import collections
 
 import itertools
 from itertools import product
 
 import numba as nb
 import numpy as np
+from sklearn.preprocessing import MinMaxScaler
 
-try:
-    import _align
-except ModuleNotFoundError:
-    import warnings
-    warnings.warn("Couldn't find cython 'align' extension, defaulting to numba implementation")
-    _align = None
-
+from ... import _align
 
 logger = logging.getLogger(__name__)
 
@@ -27,6 +23,70 @@ VGAP, HGAP, END, MATCH, UNPROCESSED = range(1, 6)
 GAP_SYM = -1
 
 
+class BaseScorer:
+    def get_scores(self, s1, s2):
+        raise NotImplementedError
+
+    def get_maximum_score(self, s1, s2):
+        raise NotImplementedError
+    
+    def fit(self, coll1, coll2, embs=None):
+        return self
+
+
+def _get_constant_scores(s1, s2, match, mismatch):
+    scores = np.zeros((len(s1), len(s2)))
+    for (i, a), (j, b) in product(enumerate(s1), enumerate(s2)):
+        scores[i, j] = match[i] if a == b else mismatch
+    return scores
+
+
+class ConstantScorer(BaseScorer):
+    def __init__(self, match=2, mismatch=-1, **kwargs):
+        self.match = match
+        self.mismatch = mismatch
+
+    def get_scores(self, s1, s2):
+        match = len(s1) * [self.match]
+        return _get_constant_scores(s1, s2, match, self.mismatch)
+
+    def get_maximum_score(self, s1, s2):
+        return self.match * min(len(s1), len(s2))
+
+
+class LookupScorer(BaseScorer):
+    def __init__(self, match=2, mismatch=-1, offset_range=(-2, 0), offset_smoothing=1.5, 
+            **kwargs):
+        self.match = match
+        self.mismatch = mismatch
+        self.offset_min, self.offset_max = offset_range
+        self.offset_smoothing = offset_smoothing
+        self.lookup = {}
+
+    def get_scores(self, s1, s2):
+        # matches can depend on the actual word (e.g. stop-words)
+        match = [self.match + self.lookup.get(w, 0) for w in s1]
+        return _get_constant_scores(s1, s2, match, self.mismatch)
+
+    def get_maximum_score(self, s1, s2):
+        if len(s1) < len(s2):
+            return sum([self.match + self.lookup.get(w, 0) for w in s1])
+        else:
+            return sum([self.match + self.lookup.get(w, 0) for w in s2])
+
+    def fit(self, coll1, coll2, **kwargs):
+        vocab, counts = zip(*collections.Counter(
+            [w for coll in [coll1, coll2] for feats in coll.get_features() for w in feats]
+        ).most_common())
+        counts = np.array(counts)
+        ranks = np.argsort(np.array(counts))[::-1]
+        scaled = MinMaxScaler(
+            feature_range=(self.offset_min, self.offset_max)
+        ).fit_transform(np.power(counts, 1/self.offset_smoothing).reshape(-1, 1)).reshape(-1)
+        self.lookup = {w: (-v + self.offset_min + self.offset_max) for w, v in zip(vocab, scaled)}
+        return self
+
+
 def _get_embedding_scores(s1, s2, d, S, match, mismatch):
     scores = np.zeros((len(s1), len(s2)))
     for (i, a), (j, b) in product(enumerate(s1), enumerate(s2)):
@@ -39,31 +99,13 @@ def _get_embedding_scores(s1, s2, d, S, match, mismatch):
     return scores
 
 
-def create_embedding_scorer(embs, match=2, mismatch=-1, **kwargs):
-    """
-    embs : retrieve.Embeddings object
-    """
-    if 'fill_missing' in kwargs:
-        raise ValueError("`fill_missing` not allowed for EmbeddingScorer")
-    keys, S = embs.get_S(**kwargs)
-    return EmbeddingScorer(keys, S, match=match, mismatch=mismatch)
-
-
-class BaseScorer:
-    def get_scores(self, s1, s2):
-        raise NotImplementedError
-
-    def get_maximum_score(self, s1, s2):
-        raise NotImplementedError
-
-
 class EmbeddingScorer(BaseScorer):
-    def __init__(self, d, S, match=2, mismatch=-1):
+    def __init__(self, match=2, mismatch=-1, field='lemma', **kwargs):
         self.match = match
         self.mismatch = mismatch
-        self.S = S
-        self.d = d
-        self.max_s = S.max()
+        self.S = self.d = self.max_s = None
+        self.field = field
+        self.kwargs = kwargs
 
     def get_scores(self, s1, s2):
         return _get_embedding_scores(s1, s2, self.d, self.S, self.match, self.mismatch)
@@ -71,30 +113,19 @@ class EmbeddingScorer(BaseScorer):
     def get_maximum_score(self, s1, s2):
         return self.match * self.max_s * min(len(s1), len(s2))
 
+    def fit(self, coll1, coll2, embs):
+        if 'fill_missing' in self.kwargs:
+            raise ValueError("`fill_missing` not allowed for EmbeddingScorer")
+        
+        from retrieve.data import get_vocab_from_colls
+        from retrieve.embeddings import Embeddings
+        vocab = get_vocab_from_colls(coll1, coll2, field=self.field)
+        embs = Embeddings.require_embeddings(embs, vocab=vocab)
+        d, S = embs.get_S(vocab=vocab, **self.kwargs)
+        self.d = d
+        self.max_s = S.max()
 
-def _get_constant_scores(s1, s2, match, mismatch):
-    scores = np.zeros((len(s1), len(s2)))
-    for (i, a), (j, b) in product(enumerate(s1), enumerate(s2)):
-        scores[i, j] = match[i] if a == b else mismatch
-    return scores
-
-
-class ConstantScorer(BaseScorer):
-    def __init__(self, match=2, mismatch=-1, manual_scores=None):
-        self.match = match
-        self.mismatch = mismatch
-        self.manual_scores = manual_scores
-
-    def get_scores(self, s1, s2):
-        # matches can depend on the actual word (e.g. stop-words)
-        if self.manual_scores is not None:
-            match = [self.manual_scores.get(w, self.match) for w in s1]
-        else:
-            match = len(s1) * [self.match]
-        return _get_constant_scores(s1, s2, match, self.mismatch)
-
-    def get_maximum_score(self, s1, s2):
-        return self.match * min(len(s1), len(s2))
+        return self
 
 
 @nb.njit
